package scorch.weapons;

/*
  Class:  Weapon
  Author: Mikhail Kruk
  Description: the class which is inherited by all the scorch weapons
  used to store explosions, prices, names etc
  Refer to README.weapon for brief descirption of weapon adding procedure
*/

import java.awt.*;
import java.awt.event.*;
import java.util.Random;
import java.lang.reflect.*;

import scorch.*;
import scorch.items.*;

public abstract class Weapon extends Item
{
    private static String pkgName = "scorch.weapons.";

	//order of types and names are important
	//assigning each weapon a number type
    public static final int Missile = 0, 
		BabyNuke = 1, Nuke = 2,
		SandBomb = 3, DirtBall=4, LiquidDirt=5,
		BabyRoller = 6, Roller = 7, HeavyRoller = 8,
		BabyDigger = 9, Digger = 10, HeavyDigger = 11, 
		FunkyBomb = 12, FunkyNuke = 13,
		Napalm = 14, HotNapalm = 15, MIRV = 16, DeathHead = 17; 

	//name must match the classname (minus the spaces)
	//to allow for instantiation
    private static final String names[] = {
	"Missile", "Baby Nuke", "Nuke",
	"Sand Bomb","Dirt Ball", "Liquid Dirt", 
	"Baby Roller", "Roller", "Heavy Roller", 
	"Baby Digger", "Digger", "Heavy Digger", 
	"Funky Bomb", "Funky Nuke",
	"Napalm", "Hot Napalm",
	"MIRV", "Death Head", 
    };

    protected String explosionClass = "weapon";
    protected int argument = -1;

    // get name of this instance of weapon
    public String getName()
    {
	return names[type];
    }

    // get weapon name by type
    public static String getName(int type)
    {
	return names[type];
    }

    // get type of the weapons buy its name
    public static int getType(String name)
    {
	for(int i = 0; i < names.length; i++)
	    if( names[i].equals(name) )
		return i;
	return -1;
    }

    public ItemControl getControlPanel(ScorchApplet owner)
    {
	controlPanel = new WeaponControl(type, owner);
	return controlPanel;
    }

    public Explosion produceExplosion(Bitmap bitmap, Random rand)
    {
	try
	    {
		// for Netscape again
		Explosion expl = (Explosion)Class.forName
		    (pkgName+explosionClass).newInstance();

		expl.setBitmap(bitmap);
		//expl.setPosition(0,0);
		expl.setRand(rand);
		expl.setArgument(argument);
		return expl;
	    }
	catch(Exception e)
	    {
		System.err.println("produceExplosion: "+e);
	    }
	
	return null;
    }

    // produce an array of all avaiable weapons. relies on the list of 
    // class names names[]. For now name of the weapons == name of the class
    // (spaces in names are omitted)
    public static Weapon[] loadWeapons(ScorchPlayer sp)
    {
	Weapon result[] = new Weapon[names.length];

	try
	    {
		for(int i = 0; i < names.length; i++)
		    {
			String name;

			if( ScorchApplet.Tournament &&
			    (i == Nuke || i == MIRV || i == FunkyNuke ||
			     i == FunkyBomb || i == DeathHead ) )
			    name = trimSpaces(pkgName, names[0]);
			else
			    name = trimSpaces(pkgName, names[i]);
			
			result[i] = (Weapon)Class.forName(name).newInstance();
			result[i].scorchPlayer = sp;
		    }
	    }
	catch(Exception e)
	    {
		System.err.println("Failed to load weapons: "+e);
	    }
	
	return result;
    }
}

// this class is a panel displayed in the inventory window to control the
// weapon. For most of them (all?) it should be just "select" button
class WeaponControl extends ItemControl implements ActionListener
{
    protected int type;

    public WeaponControl(int type, ScorchApplet owner)
    {
	super(null, owner);
	this.type = type;
	
        control = new Button("Select");
	((Button)control).addActionListener(this);
	
	setLayout(new FlowLayout(FlowLayout.CENTER));
	add(control);
    }

    public void actionPerformed(ActionEvent evt)
    {
	if( evt.getActionCommand().equals("Select") )
	    {
		owner.selectWeapon(type);
	    }
    }
}
