/*
  Class:  ScorchServer

  Author: Mikhail Kruk
  
  Periodically dump server statistics in HTML format
*/

package scorchserver;

import java.util.*;
import java.io.*;

public class ServerStats implements Runnable
{
    private ScorchServer owner;
    private Thread thread;
    private static int DELAY = 1000*60*2;
    private static String fileName = "stat.html";
    private boolean running = true;

    private final static String HTML_START = 
	"<!doctype html public \"-//w3c//dtd html 4.0 transitional//en\">\n"+
"<html>\n"+
"<head>\n"+
"   <meta http-equiv=\"Content-Type\" content=\"text/html\">\n"+
"   <TITLE>Scorched Earth 2000 Server Status</TITLE>\n"+
"</head>\n"+
"<body>\n"+
"<center>\n"+
"<h2>Server Stats</h2>\n"+
"<TABLE border=\"1\" borderColor=\"black\" cellSpacing=\"0\" cellPadding=\"5\">";
    private final static String HTML_END =
	"</TABLE>\n"+
"</center>\n"+
"<p><a href=\"index.html\">back</a>\n"+
"<br><a href=\"mailto:scorch@scorch2000.com\">contact</a>\n"+
"</body>\n"+
"</html>";


    public ServerStats(ScorchServer owner)
    {
	this.owner = owner;
	thread = new Thread(this, "server stats");
	thread.start();
    }

    public void shutdown()
    {
	running = false;
    }

    public void run()
    {
	while(running)
	    {
		try
		    {
			FileOutputStream fout = new FileOutputStream(fileName);
			BufferedOutputStream out = 
			    new BufferedOutputStream(fout);
			
			out.write(HTML_START.getBytes());
			writeRow(out, "Server version", ScorchServer.version);
			writeRow(out, "Uptime", ScorchServer.getUpTime());
			writeRow(out, "Users online", 
				 ""+ScorchServer.getPlayersCount());
			writeRow(out, "Active games", 
				 ScorchServer.allGamesToString(false));
			writeRow(out, "Total games", ScorchServer.getGameCount()+
				 " ("+ScorchServer.potentialDesyncCount+")");
			writeRow(out, "Peak usage", 
				ScorchServer.getMaxGamesCount());
			writeRow(out, "Registered users", 
				""+owner.getRegisteredUsersCount());
			//writeRow(out, "Last upated", (new Date()).toString());
			out.write(HTML_END.getBytes());
			out.flush();
			out.close();
			fout.close();
		    }
		catch(IOException e)
		    {
			System.err.println(e);
		    }
		
		try
		    {
			Thread.sleep(DELAY);
		    }
		catch(InterruptedException e) {}
	    }
    }

    private void writeRow(BufferedOutputStream out, 
			  String label, String value) throws IOException
    {
	String res = "";
	res += "<TR><TD>";
	res += label;
	res += "</TD><TD>";
	res += value;
	res += "</TD></TR>\n";
	out.write(res.getBytes());
    }
}
